import 'antd/dist/antd.css';
import React, { useContext, useEffect, useState } from 'react';

import { useUnauthRedirect } from '../../../hooks';
import { useRouter } from 'next/router';
import { Button, Table, Typography } from 'antd';
import { faFileLines, faUser } from '@fortawesome/free-solid-svg-icons';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { LoggedUserContext } from '../../../contexts/LoggedUserContext';
import { MainLayout } from '../../../layouts/MainLayout';
import AddDocumentModal from '../../../components/modals/AddDocumentModal';
import {
    DocumentListInfo,
    DocumentListResponse,
    DocumentUserInfo,
    EState,
} from '../../../api';
import { documentController } from '../../../controllers';
import AssignDocumentModal from '../../../components/modals/AssignDocumentModal';
import { ShowToast } from '../../../utils/alerts';
import { TableDocInfo } from '../../../components/types/TableDocInfo';
import {
    getAnnotationStateColor,
    getNameTruncated,
    getUserInfoAlt,
} from '../../../utils/strings';
import { ABadge, BadgeStyle } from '../../../components/common/ABadge';

function AdminDocumentPage() {
    const redirecting = useUnauthRedirect('/login');
    const { logout, role } = useContext(LoggedUserContext);
    const [visibleAdd, setVisibleAdd] = React.useState(false);
    const [visibleAssign, setVisibleAssign] = React.useState(false);
    const router = useRouter();

    const [documents, setDocuments] = useState<TableDocInfo[]>([]);
    const [selectedDocs, setSelectedDocs] = useState<string[] | undefined[]>([]);

    async function fetchData() {
        const docs = (await documentController.documentsGet(0, 1000)).data.documents;

        // @ts-ignore
        const tableDocs: TableDocInfo[] = docs?.map((doc, index) => {
            return { key: index, ...doc };
        });

        if (!docs) {
            setDocuments([]);
        } else {
            setDocuments(tableDocs);
        }
    }

    useEffect(() => {
        if (!redirecting && role === 'ADMINISTRATOR') {
            fetchData();
        }
    }, [logout, redirecting, role, router]);

    const showAssignModal = () => {
        if (selectedDocs.length == 0) {
            ShowToast('Vyberte dokument pro přiřazení', 'warning', 3000, 'top-end');
        } else {
            setVisibleAssign(true);
        }
    };

    const showAddModal = () => {
        setVisibleAdd(true);
    };

    const hideModal = () => {
        fetchData();
        setVisibleAdd(false);
        setVisibleAssign(false);
    };

    const columns = [
        {
            title: 'Název dokumentu',
            dataIndex: 'name',
            key: 'name',
        },
        {
            title: 'Délka',
            dataIndex: 'length',
            key: 'length',
        },
        {
            title: 'Dokončeno | přiřazeno | vyžadováno',
            key: 'annotationCounts',
            render: (
                columnData: DocumentListResponse,
                record: DocumentListInfo,
                index: number
            ) => {
                const finished =
                    record.annotatingUsers?.filter((d) => d.state === EState.Done)
                        .length ?? 0;

                return (
                    <div>
                        <ABadge
                            style={
                                finished === record.annotatingUsers?.length
                                    ? BadgeStyle.SUCCESS
                                    : BadgeStyle.WARNING
                            }
                        >
                            {finished}
                        </ABadge>
                        {' | '}
                        <ABadge
                            style={
                                record.annotatingUsers?.length ===
                                record.requiredAnnotations
                                    ? BadgeStyle.SUCCESS
                                    : BadgeStyle.WARNING
                            }
                        >
                            {record.annotatingUsers?.length}
                        </ABadge>
                        {' | '}
                        <ABadge style={BadgeStyle.GENERAL}>
                            {record.requiredAnnotations}
                        </ABadge>
                    </div>
                );
            },
        },
        {
            title: 'Anotátoři',
            dataIndex: 'annotatingUsers',
            key: 'annotatingUsers',
            render: (
                columnData: DocumentUserInfo[],
                record: DocumentListInfo,
                index: number
            ) => {
                return (
                    <div>
                        {columnData.map((e) => (
                            <span
                                key={e.username + '.' + record.id}
                                title={getUserInfoAlt(e) + '\nStav: ' + e.state}
                                style={{
                                    color: getAnnotationStateColor(e.state),
                                    padding: 3,
                                }}
                                className={'me-3'}
                            >
                                <FontAwesomeIcon
                                    icon={faUser}
                                    title={getUserInfoAlt(e)}
                                    className={'me-2'}
                                />
                                {getNameTruncated(e)}
                            </span>
                        ))}
                    </div>
                );
            },
        },
    ];

    const rowSelection = {
        onChange: (selectedRowKeys: React.Key[], selectedRows: DocumentListInfo[]) => {
            // @ts-ignore
            setSelectedDocs(selectedRows.map((row) => row.id));
        },
    };

    return redirecting || role !== 'ADMINISTRATOR' ? null : (
        <MainLayout>
            <Typography.Title level={2}>
                <FontAwesomeIcon icon={faFileLines} /> Dokumenty
            </Typography.Title>
            <Button type={'primary'} onClick={showAddModal}>
                Nahrát dokument
            </Button>
            <Button onClick={showAssignModal}>Přiřadit dokumenty</Button>
            {visibleAdd && <AddDocumentModal onCancel={hideModal} />}
            {visibleAssign && (
                <AssignDocumentModal documentsIds={selectedDocs} onCancel={hideModal} />
            )}

            <Table
                rowSelection={{
                    type: 'checkbox',
                    ...rowSelection,
                }}
                columns={columns}
                dataSource={documents}
                size="middle"
                scroll={{ y: 600 }}
            />
        </MainLayout>
    );
}

export default AdminDocumentPage;
